const PACKAGES = ['@talend/react-components', 'lodash'];

function find(path) {
	return PACKAGES.find(p => path.node.source.value.startsWith(p));
}

function getLastNameInPath(path) {
	const splited = path.split('/');
	if (splited.length === 1) {
		return false;
	}
	return splited.pop();
}

module.exports = function transform({ types }) {
	return {
		visitor: {
			Program(path) {
				const importSpecifiers = {};
				let lastImport;

				path.traverse({
					ImportDeclaration: {
						exit(importDeclarationPath) {
							const packageName = find(importDeclarationPath);
							if (packageName && !importSpecifiers[packageName]) {
								importSpecifiers[packageName] = [];
							}
							if (packageName) {
								importSpecifiers[packageName].push(
									...importDeclarationPath.node.specifiers.map(({ local, imported, type }) => {
										const localName = local.name;
										const lastName = getLastNameInPath(importDeclarationPath.node.source.value);

										if (type === 'ImportNamespaceSpecifier') {
											return types.importNamespaceSpecifier(types.identifier(localName));
										}

										let importedName = localName;
										if (imported) {
											// case {}
											importedName = imported.name;
										} else if (lastName && lastName !== localName && packageName === 'lodash') {
											importedName = lastName;
										}
										return types.importSpecifier(
												types.identifier(localName),
												types.identifier(importedName),
											);
									}),
								);
								importDeclarationPath.remove();
							}
							lastImport = importDeclarationPath;
						},
					},
				});
				const keys = Object.keys(importSpecifiers);
				if (keys.length > 0) {
					keys.forEach(packageName => {
						const source = types.stringLiteral(packageName);
						const imp = types.importDeclaration(importSpecifiers[packageName], source);
						lastImport.insertAfter(imp);
					});
				}
			},
		},
	};
};
